package gov.anl.aps.dm.portal.controllers;

import gov.anl.aps.dm.common.exceptions.DmException;
import gov.anl.aps.dm.common.exceptions.InvalidRequest;
import gov.anl.aps.dm.common.exceptions.ObjectAlreadyExists;
import gov.anl.aps.dm.portal.model.beans.ExperimentTypeDbFacade;
import gov.anl.aps.dm.portal.model.entities.ExperimentType;
import javax.ejb.EJB;
import javax.inject.Named;
import javax.enterprise.context.SessionScoped;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.FacesConverter;
import org.apache.log4j.Logger;

@Named("experimentTypeController")
@SessionScoped
public class ExperimentTypeController extends DmEntityController<ExperimentType, ExperimentTypeDbFacade> {

    private static final Logger logger = Logger.getLogger(ExperimentTypeController.class.getName());

    @EJB
    private ExperimentTypeDbFacade experimentTypeDbFacade;

    public ExperimentTypeController() {
    }

    @Override
    protected ExperimentTypeDbFacade getFacade() {
        return experimentTypeDbFacade;
    }

    @Override
    protected ExperimentType createEntityInstance() {
        return new ExperimentType();
    }

    @Override
    public String getEntityTypeName() {
        return "experimentType";
    }

    @Override
    public String getCurrentEntityInstanceName() {
        if (getCurrent() != null) {
            return getCurrent().getName();
        }
        return "";
    }

    @Override
    public ExperimentType findById(Integer id) {
        return experimentTypeDbFacade.findById(id);
    }
    
    @Override
    public void prepareEntityInsert(ExperimentType experimentType) throws DmException {
        if ((experimentType.getName() == null) || (experimentType.getName().length() == 0)) {
            throw new InvalidRequest("Experiment type name is missing.");
        }
        ExperimentType existingExperimentType = experimentTypeDbFacade.findByName(experimentType.getName());
        if (existingExperimentType != null) {
            throw new ObjectAlreadyExists("Experiment type " + experimentType.getName() + " already exists.");
        }
        logger.debug("Inserting new experiment type " + experimentType.getName());
    }

    @Override
    public void prepareEntityUpdate(ExperimentType experimentType) throws DmException {
        if ((experimentType.getName() == null) || (experimentType.getName().length() == 0)) {
            throw new InvalidRequest("Experiment type name is missing.");
        }
        logger.debug("Updating experiment type " + experimentType.getName());
    }

    @Override
    protected String getObjectAlreadyExistMessage(ExperimentType experimentType) {
        if (experimentType == null) {
            return null;
        }
        return "Experiment type " + experimentType.getName() + " already exists.";
    }

    @FacesConverter(forClass = ExperimentType.class)
    public static class ExperimentTypeControllerConverter implements Converter {

        @Override
        public Object getAsObject(FacesContext facesContext, UIComponent component, String value) {
            if (value == null || value.length() == 0) {
                return null;
            }
            ExperimentTypeController controller = (ExperimentTypeController) facesContext.getApplication().getELResolver().
                    getValue(facesContext.getELContext(), null, "experimentTypeController");
            return controller.getEntity(getKey(value));
        }

        java.lang.Integer getKey(String value) {
            java.lang.Integer key;
            key = Integer.valueOf(value);
            return key;
        }

        String getStringKey(java.lang.Integer value) {
            StringBuilder sb = new StringBuilder();
            sb.append(value);
            return sb.toString();
        }

        @Override
        public String getAsString(FacesContext facesContext, UIComponent component, Object object) {
            if (object == null) {
                return null;
            }
            if (object instanceof ExperimentType) {
                ExperimentType o = (ExperimentType) object;
                return getStringKey(o.getId());
            } else {
                throw new IllegalArgumentException("Object " + object + " is of type " + object.getClass().getName() + "; expected type: " + ExperimentType.class.getName());
            }
        }

    }

}
