#!/usr/bin/env python

from dm.cat_web_service.api.datasetRestApi import DatasetRestApi
from dm.common.exceptions.invalidRequest import InvalidRequest
from catWebServiceSessionCli import CatWebServiceSessionCli

class AddExperimentDatasetCli(CatWebServiceSessionCli):
    def __init__(self):
        CatWebServiceSessionCli.__init__(self, validArgCount=self.ANY_NUMBER_OF_POSITIONAL_ARGS)
        self.addOption('', '--dataset', dest='datasetName', help='Dataset name.')
        self.addOption('', '--experiment', dest='experimentName', help='Experiment name.')

    def checkArgs(self):
        if self.options.experimentName is None:
            raise InvalidRequest('Experiment name must be provided.')
        if self.options.datasetName is None:
            raise InvalidRequest('Dataset name must be provided.')

    def getExperimentName(self):
        return self.options.experimentName

    def getDatasetName(self):
        return self.options.datasetName

    def runCommand(self):
        self.parseArgs(usage="""
    dm-add-experiment-dataset --dataset=DATASETNAME --experiment=EXPERIMENTNAME 
        [key1:value1, key2:value2, ...]

Description:
    Adds experiment dataset to the metadata catalog. All provided key/value 
    pairs are interpreted as dataset metadata.
        """)
        self.checkArgs()
        api = DatasetRestApi(self.getLoginUsername(), self.getLoginPassword(), self.getServiceHost(), self.getServicePort(), self.getServiceProtocol())
        datasetInfo = self.splitArgsIntoDict()
        datasetInfo['experimentName'] = self.getExperimentName()
        datasetInfo['datasetName'] = self.getDatasetName()
        datasetMetadata = api.addExperimentDataset(datasetInfo)
        print datasetMetadata.getDisplayString(self.getDisplayKeys(), self.getDisplayFormat())

#######################################################################
# Run command.
if __name__ == '__main__':
    cli = AddExperimentDatasetCli()
    cli.run()

