#!/usr/bin/env python

from daqWebServiceSessionCli import DaqWebServiceSessionCli

from dm.common.constants import dmProcessingStatus
from dm.daq_web_service.api.experimentRestApi import ExperimentRestApi
from dm.common.exceptions.invalidRequest import InvalidRequest

class ListUploadsCli(DaqWebServiceSessionCli):

    def __init__(self):
        DaqWebServiceSessionCli.__init__(self, validArgCount=self.ANY_NUMBER_OF_POSITIONAL_ARGS)
        self.addOption('', '--status', dest='status', default=dmProcessingStatus.DM_PROCESSING_STATUS_ANY, help='Processing status, must be one of %s (default: %s).' % (dmProcessingStatus.DM_ALLOWED_PROCESSING_STATUS_LIST,dmProcessingStatus.DM_PROCESSING_STATUS_ANY))

    def checkArgs(self):
        if self.options.status not in dmProcessingStatus.DM_ALLOWED_PROCESSING_STATUS_LIST:
            raise InvalidRequest('Processing status must be one of %s.' % dmProcessingStatus.DM_ALLOWED_PROCESSING_STATUS_LIST)

    def getStatus(self):
        return self.options.status

    def runCommand(self):
        self.parseArgs(usage="""
    dm-list-uploads [--status=STATUS]

Description:
    Retrieves all known uploads.
        """)
        self.checkArgs()
        api = ExperimentRestApi(self.getLoginUsername(), self.getLoginPassword(), self.getServiceHost(), self.getServicePort(), self.getServiceProtocol())
        uploads = api.listUploads(self.getStatus())
        for upload in uploads:
            print upload.getDisplayString(self.getDisplayKeys(), self.getDisplayFormat())

#######################################################################
# Run command.
if __name__ == '__main__':
    cli = ListUploadsCli()
    cli.run()

